/*
	Programma di esempio che utilizza le classi MFC DAO
	per accedere ad un database Access.

	Copyright 2000 Davide Bianchi - GPL

	Gnu GPL - General Public License
	Questo programma e tutta la documentazione accompagnatoria sono da considerarsi
	coperti da licenza secondo i dettami della GNU GPL - General Public License, in
	breve potete copiare, riprodurre, modificare ed utilizzare il codice e la
	documentazione allegata come piu' vi pare purche' forniate a chiunque altro gli
	stessi diritti su tutte le eventuali copie modificate di questo codice, inoltre
	lasciate intatta l'attuale licenza.

	Potete trovare ulteriori informazioni relative alla GPL su http://www.fsf.org

*/

#define WIN_LEAN_AND_MEAN	/* rimuove un sacco di schifezza */

#include <stdlib.h>
#include <string.h>
#include <afx.h>
#include <afxdao.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "dao.h"

/*********************************************************************************

  MAIN

  Il programma si avvia richiamando DAO <database>

  Dove <database> e' il nome che si vuole assegnare allo stesso database.
  Se non indicato, verra' creato un database "standard" denominato DAO.MDB.
  Nel caso in cui il database esista gia', verra' utilizzato.

*********************************************************************************/

int main( int argc, char *argv[] )
{

  char database[255];                       /* nome del database */
  CDaoDatabase *daoDb = new CDaoDatabase(); /* oggetto Database */
  BOOL esiste = FALSE;                      /* flag se il database esiste o no */
  struct _stat buf;                         /* solo per verificare l'esistenza del db */

  /* inizializzo il motore DAO */
  AfxDaoInit();
  
	/* controllo se ho passato il nome */
  if( argc < 2 )
  {
    /* nessun nome, assegno il default */
    strcpy( database, DATABASE_NAME );
  }
  else
  {
    /* leggo il nome indicato */
    strcpy( database, argv[1] );
  }

  /* verifico se il database esiste o meno */
  if( _stat( database, &buf) == 0 )
  {
    /* apro il database, lascio tutti i parametri di default */
    daoDb->Open( database );

    /* segnalo che il database esiste */
    esiste = TRUE;
  }
  else
  {
    /* il database non esiste, lo creo, lascio tutti i parametri di default */
    daoDb->Create( database );
  }

  /*
    verifico se il database esisteva gia' o l'ho creato adesso,
    se l'ho creato adesso devo anche creare le tabelle interne...
  */
  if( !esiste )
  {
    /* esegue la creazione delle tabelle */
    if( creaTabella( daoDb ) )
	  {
		  /* errore nella creazione */
		  daoDb->Close();
		  return(0);
	  }
  }

  /* a questo punto ho un database aperto ed attivo */
  mostraPrompt( daoDb );

  /* chiudo il database */
  daoDb->Close();
  
  /*
    chiudo il WorkSpace, questo ha l'effetto di distruggere anche
    il database
  */
  daoDb->m_pWorkspace->Close();

  /*
  ATTENZIONE: se il motore di database non viene terminato si crea
  un GPF che porta anche all'abort di Visual Studio.
  */
  AfxDaoTerm( );

  return(0);

}

/*********************************************************************************

  mostraPrompt

  Questa funzione viene usata per la visualizzazione del prompt all'utente.

*********************************************************************************/
char mostraPrompt( CDaoDatabase *daoDb )
{
  char c;
  int  i;

  do
  {
    printf("\nDAO> ");
    scanf( "%c", &c);

    /* verifico il comando premuto */
    switch( c )
    {
      case 'q':
      case 'Q':
        /* Quit */
        break;

      case 'a':
      case 'A':
        /* Aggiungi righe */
        aggiungiRecord( daoDb );
        break;

      case 'l':
      case 'L':
        /* lista righe */
        leggiRecord( daoDb );
        break;

      case 'd':
      case 'D':
        /* cancella righe */
        printf("\nriga da cancellare: " );
        scanf("%d", &i );
        cancellaRecord( daoDb, i );
        break;

      case '?':
        /* help */
        printf("Comandi:\n");
        printf("a = aggiunge righe alla tabella\n");
        printf("d = elimina righe dalla tabella\n");
        printf("l = lista le righe presenti\n");
        printf("q = fine programma\n");
        printf("? = visualizza questo schermo.\n");
        printf("\n");
        break;
    }

  } while( c != 'q' && c != 'Q' );

  /* ritorno il comando premuto */
  return(c);

}



/*********************************************************************************

  creaTabelle

  Questa funzione viene usata per la creazione di una tabella di prova
  all'interno del database se questo e' stato appena creato.

  La funzione costruisce una tabella denominata Anagrafica, con la seguente
  struttura:

  ID      contatore-chiave univoca
  Nome    Stringa 70 
  Cognome Stringa 70
  Eta     Numerico

*********************************************************************************/
int creaTabella( CDaoDatabase *daoDb )
{

  int ret=0;
  char *sql = (char*) malloc(1024);

  strcpy( sql, "CREATE TABLE Anagrafica (ID Number, Nome Text(70), Cognome Text(70), Eta Number );");

  try
  {
    daoDb->Execute( sql );
  }
  catch( CDaoException* )
  {
    printf("Errore nella creazione delle tabelle: " );
	ret=1;
  }
  
  /* libero memoria */
  free( sql );
  return( ret );
}

/*********************************************************************************

  inserisciRecord

  Questa funzione serve ad inserire un record nel database.

*********************************************************************************/
void inserisciRecord( int id, char *nome, char *cognome, int eta, CDaoDatabase *daoDb )
{
  char *sql = (char*) malloc( 1024 );

  sprintf( sql, "INSERT INTO Anagrafica (ID, Nome, Cognome, Eta) VALUES (%d, '%s','%s',%d);", id, nome, cognome, eta ) ;
  daoDb->Execute( sql );

  free( sql );
  return;
}

/*********************************************************************************

  aggiungiRecord

  Questa funzione serve a richiedere i dati del record da
  aggiungere.

*********************************************************************************/
void aggiungiRecord( CDaoDatabase *daoDb )
{
  char nome[50];
  char cognome[50];
  int eta;
  int id;

  printf("\nNome: ");
  scanf( "%s", nome );
  printf("\nCognome: ");
  scanf( "%s", cognome );
  printf("\nEta: " );
  scanf( "%d", &eta );
  printf("\nID: ");
  scanf("%d", &id );

  inserisciRecord( id, nome, cognome, eta, daoDb );
  return;

}

/*********************************************************************************

  cancellaRecord

  Questa funzione viene usata per eliminare i record dalla tabella
*********************************************************************************/
void cancellaRecord( CDaoDatabase *daoDb, int numRec )
{
  char *sql = (char*) malloc( 1024 );
  
  sprintf( sql, "DELETE FROM ANAGRAFICA WHERE ID=%d;", numRec );
  daoDb->Execute( sql );

  free( sql );

}


/*********************************************************************************

  leggiRecord

  Questa funzione legge i record presenti nella tabella e visualizza
  le informazioni a video.

*********************************************************************************/
void leggiRecord( CDaoDatabase *daoDb )
{

  CDaoRecordset recordset(daoDb);       /* recordset usato per leggere */
  char *sql = (char*) malloc( 1024 );   /* stringa SQL */
  COleVariant v;                        /* tutti i campi sono VARIANT */
  int i;                                /* contatore per scorrere i campi */
  CString campo;                        /* usato per i campi di testo */
  double val;                           /* usato per i campi numerici */
  struct CDaoFieldInfo info;            /* informazioni sui campi */


  /* apro il recordset solo in lettura, non mi serve altro */
  strcpy( sql, "SELECT ID, Nome, Cognome, Eta From Anagrafica ORDER BY ID;");
  recordset.Open( dbOpenDynaset, sql, dbReadOnly );

  /* ciclo su tutti i record e li visualizzo */
  while( ! recordset.IsEOF() )
  {
    /* ciclo e reperisco tutti i campi del record */
    for( i=0; i < recordset.GetFieldCount(); i++ )
    {
      /* reperisco il campo corrente */
      recordset.GetFieldValue( i, v );

      /* verifico il tipo di campo */
      recordset.GetFieldInfo( i, info );

      /* che tipo di campo ho ? */
      if( info.m_nType == dbInteger || info.m_nType == dbSingle || info.m_nType == dbDouble )
      {
        /* e' un valore numerico */
        val = V_R8(&v);
        printf( "%4.0f\t", val );
      }

      if( info.m_nType == dbText )
      {
        /* e' una stringa */
        campo = V_BSTRT( &v );
        printf( "%s\t", campo );
      }

    }

    /* vado a capo */
    printf("\n");

    /* prossimo record */
    recordset.MoveNext( );

  }

  /* chiudo il recordset */
  recordset.Close();
  free( sql );

  return;
}

